/*
    Copyright  1995-2001, The AROS Development Team. All rights reserved.
    $Id: prefsupdate.c,v 1.6 2018/02/21 19:41:08 piru Exp $

    Desc: Tell locale that the preferences have been changed.
    Lang: english
*/

#include <exec/types.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <dos/var.h>
#include <utility/utility.h>
#include "locale_intern.h"

#include <string.h>

static const char * const langlist[] =
{
	"english",
	"deutsch",
	"franais",
	"espaol",
	"italiano",
	"portugus",
	"dansk",
	"nederlands",
	"norsk",
	"suomi",
	"svenska",
	NULL
};

#ifdef __MORPHOS__

#include <stdint.h>
#include <exec/execbase.h>
#include <hardware/byteswap.h>

#pragma pack(4)
struct ixtime
{
	int32_t gmtoffset; /* GMT offset in seconds */
	uint8_t flags;     /* bit 0 set if daylight saving time (unused) */
};
#pragma pack()


static void ixemul_update_gmt_offset(struct LocaleBase *LocaleBase, int32_t offset)
{
	const struct ixtime t = {LE_SWAPLONG(offset), 0};
	ULONG oldsignals;

	/* Update the IXGMTOFFSET environment variable */
	SetVar("IXGMTOFFSET", (APTR) &t, 5, LV_VAR | GVF_GLOBAL_ONLY);

	/* If ixemul.library is in use, call ix_set_gmt_offset to set the offset */

	Forbid();
	/* Make sure no signals are lost - ixemul libopen does some random shit. */
	oldsignals = SetSignal(0, 0);
	if (FindExecNode(EXECLIST_LIBRARY, "ixemul.library"))
	{
		struct Library *IXEmulBase;
		UBYTE emptybstr[4];
		struct CommandLineInterface fakecli;

		/* slight hack: make sure that ixemul libopen doesn't get stuck waiting
		 * for Ambient startup message...
		 */
		struct Process *self = (APTR) FindTask(NULL);
		BPTR oldcli = self->pr_CLI;
		if (oldcli == 0) /* NOTE: Is always 0 really since the caller is the IPrefs process...*/
		{
			/* Really not used by ixemul libopen itself, but in case something
			 * else actually peeks the structure provide a fake one...
			 */
			memset(&fakecli, 0, sizeof(fakecli));
			memset(emptybstr, 0, sizeof(emptybstr));
			fakecli.cli_SetName = MKBADDR(emptybstr);
			fakecli.cli_CommandName = MKBADDR(emptybstr);
			fakecli.cli_Prompt = MKBADDR(emptybstr);
			self->pr_CLI = MKBADDR(&fakecli);
		}
		IXEmulBase = OpenLibrary("ixemul.library", 42);
		self->pr_CLI = oldcli;
		if (IXEmulBase)
		{
			/* SYSTEM_CALL (ix_set_gmt_offset, 474, V_I, STK) */
			REG_A6 = (ULONG) IXEmulBase;
			REG_A7 -= sizeof(ULONG) * 2;
			((ULONG *) REG_A7)[0] = (ULONG) ixemul_update_gmt_offset; /* return address (unused) */
			((ULONG *) REG_A7)[1] = (ULONG) offset;                   /* arg1 */
			(void) (*MyEmulHandle->EmulCallDirectOS)((474 + 4) * -6);
			REG_A7 += sizeof(ULONG) * 2;

			CloseLibrary(IXEmulBase);
		}
	}
	SetSignal(oldsignals, 0xffffffff);
	Permit();
}

#endif

/*****i***********************************************************************

    NAME */
#include <proto/locale.h>

AROS_LH1(struct Locale *, LocalePrefsUpdate,

         /*  SYNOPSIS */
         AROS_LHA(struct Locale  *, locale, A0),

         /*  LOCATION */
         struct LocaleBase *, LocaleBase, 28, Locale)

/*  FUNCTION
	This internal function is called by the IPrefs program to
	notify the locale.library that the system preferences have
	been changed.

    INPUTS
	locale	-	A pointer to the new locale preferences.

    RESULT
	The address of the old locale preferences data. This should
	have CloseLocale() called upon it.

    NOTES
	You can safely call CloseLocale() on the new Locale as this
	function will ensure that the Locale will not be freed until
	PrefsUpdate() is called again.

    EXAMPLE

    BUGS

    SEE ALSO

    INTERNALS

    HISTORY
	27-11-96    digulla automatically created from
			    locale_lib.fd and clib/locale_protos.h

*****************************************************************************/
{
	AROS_LIBFUNC_INIT
	AROS_LIBBASE_EXT_DECL(struct Library *, LocaleBase)

	STRPTR language, dotptr;
	APTR old = NULL;
	ULONG index = 0;

	if(locale != NULL)
	{
		/* Lock the locale structures */
		ObtainSemaphore(&IntLB(LocaleBase)->lb_LocaleLock);

		old = IntLB(LocaleBase)->lb_CurrentLocale;

		/* Update the count and install the new locale. */
		IntL(locale)->il_Count++;
		IntLB(LocaleBase)->lb_CurrentLocale = IntL(locale);
		ReleaseSemaphore(&IntLB(LocaleBase)->lb_LocaleLock);

		/* Let's be mean to save some hassle */
		language = locale->loc_LanguageName;
		if((dotptr = strchr(language, '.'))) * dotptr = 0;

		/* Update ENV:Language */
		SetVar("Language", language, -1, LV_VAR | GVF_GLOBAL_ONLY);

#ifdef __MORPHOS__
		/* update ixemul GMT offset (seconds) from loc_GMTOffset (minutes) */
		ixemul_update_gmt_offset(LocaleBase, locale->loc_GMTOffset * 60);
#endif

		/* Update UtilityBase->ub_Language */
		while(langlist[index] != NULL)
		{
			if(strcmp(language, langlist[index++]) == 0)
			{
				/* Check for american-english */
				if(index == 1 && locale->loc_MeasuringSystem == MS_AMERICAN) --index;

#undef UtilityBase
				((struct UtilityBase *)IntLB(LocaleBase)->lb_UtilityBase)->ub_Language = index + 1;
				break;
			}
		}

		if(dotptr) *dotptr = '.';
	}
	return (struct Locale *)old;

	AROS_LIBFUNC_EXIT
} /* PrefsUpdate */
